/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Manipulate data types on the bit (or byte) level.
 */


#include <jwsc/config.h>

#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>

#include "jwsc/base/bits.h"


/**
 * @name bswap
 *
 * Swaps the byte ordering between big and little endian.
 * @{
 */

/**
 * Byte swaps a big-endian value to little-endian and vise versa. the parameter 
 * is passed by reference.
 *
 * @param  x  Value to swap the bytes of.
 */
void bswap_u16(uint16_t* x)
{
    *x = (*x>>8) | 
         (*x<<8);
}


/**
 * Byte swaps a big-endian value to little-endian and vise versa. the parameter 
 * is passed by reference.
 *
 * @param  x  Value to swap the bytes of.
 */
void bswap_u32(uint32_t* x)
{
    *x = (*x>>24) | 
         ((*x<<8) & 0x00FF0000) |
         ((*x>>8) & 0x0000FF00) |
         (*x<<24);
}


/**
 * Byte swaps a big-endian value to little-endian and vise versa. the parameter 
 * is passed by reference.
 *
 * @param  x  Value to swap the bytes of.
 */
void bswap_u64(uint64_t* x)
{
    uint64_t a = 0xFF;

    *x = (*x>>56) | 
         ((*x<<40) & (a<<48)) |
         ((*x<<24) & (a<<40)) |
         ((*x<<8)  & (a<<32)) |
         //((*x<<40) & 0x00FF000000000000) |
         //((*x<<24) & 0x0000FF0000000000) |
         //((*x<<8)  & 0x000000FF00000000) |
         ((*x>>8)  & 0x00000000FF000000) |
         ((*x>>24) & 0x0000000000FF0000) |
         ((*x>>40) & 0x000000000000FF00) |
         (*x<<56);
}

/*@}*/
