/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Manipulate data types on the bit (or byte) level.
 */


#ifndef BITS_H
#define BITS_H


#include <jwsc/config.h>

#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>


#ifdef __cplusplus
namespace jwsc {
extern "C" {
#endif


/**
 * @name bswap
 *
 * Swaps the byte ordering between big and little endian.
 * @{
 */

/** @brief Swaps the 16-bit byte ordering between big and little endian. */
void bswap_u16(uint16_t* x);


/** @brief Swaps the 32-bit byte ordering between big and little endian. */
void bswap_u32(uint32_t* x);


/** @brief Swaps the 64-bit byte ordering between big and little endian. */
void bswap_u64(uint64_t* x);

/*@}*/


#ifdef __cplusplus
}
}
#endif


#endif
