/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Declarations for manipulating file and directory names.
 */


#ifndef FNAME_H
#define FNAME_H


#include <jwsc/config.h>

#include <stdlib.h>
#include <stdio.h>


#ifdef __cplusplus
namespace jwsc {
extern "C" {
#endif


/** @brief Returns whether a file name has a certain suffix. */
int fname_has_suffix(const char* fname, const char* suffix);


/** @brief Returns a copy of the file name suffix, if it has one. */
char* get_fname_suffix(const char* fname);


/** @brief Returns a copy of the file name directory prefix. */
char* get_fname_dir(const char* fname);


/* @brief Returns a copy fo the base file name without directories. */
char* get_fname_base(const char* fname);


#ifdef __cplusplus
}
}
#endif


#endif
