/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Read, write, and access XML documents.
 *
 * The XML data structures are documented at http://xmlsoft.org/html
 */


#ifndef XML_H
#define XML_H


#include <jwsc/config.h>

#include <stdlib.h>
#include <stdio.h>

#ifdef JWSC_HAVE_LIBXML2
#include <libxml/tree.h>
#endif

#include "jwsc/base/error.h"


#ifdef __cplusplus
namespace jwsc {
extern "C" {
#endif


/**
 * @struct xmlDoc
 *
 * @brief XML document.
 *
 * Documented at http://xmlsoft.org/html/libxml-tree.html#xmlDoc
 */


/**
 * @struct xmlDtd
 *
 * @brief XML document type definition.
 *
 * Documented at http://xmlsoft.org/html/libxml-tree.html#xmlDtd
 */


/** @brief Creates an empty xml document. */
void create_xml_doc(xmlDoc** doc_out);


/** @brief Frees an xml document. */
void free_xml_doc(xmlDoc* doc);


/** @brief Reads an xml document from a file. */
Error* read_xml_doc(xmlDoc** doc_out, const char* fname);


/** @brief Reads an xml dtd from a file. */
Error* read_xml_dtd(xmlDtd** dtd_out, const char* fname);


/** @brief Frees an xml dtd. */
void free_xml_dtd(xmlDtd* dtd);


/** @brief Validates an xml document against a dtd. */
int validate_xml_doc(xmlDoc* doc, xmlDtd* dtd);


#ifdef __cplusplus
}
}
#endif


#endif
