/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/** 
 * @file 
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Declarations for functions to read and write images.
 */


#ifndef IMAGE_IO_H
#define IMAGE_IO_H


#include <jwsc/config.h>

#include <stdlib.h>

#include "jwsc/base/error.h"
#include "jwsc/image/image.h"


#ifdef __cplusplus
namespace jwsc {
extern "C" {
#endif


/** 
 * @name read_image
 *
 * Reads an image from a file.
 * @{
 */

/** @brief Reads a single precision floating point image from a file. */
Error* read_image_f(Image_f** img_out, const char* fname);

/*@}*/




/** 
 * @name read_image_with_magick_f
 *
 * Reads an image from a file using ImageMagick.
 * @{
 */

/** 
 * @brief Reads a single precision floating point image from a file using
 * ImageMagick.
 */
#ifdef JWSC_HAVE_MAGICK
Error* read_image_with_magick_f(Image_f** img_out, const char* fname);
#endif

/*@}*/




/** 
 * @name write_image
 *
 * Writes an image to a file.
 * @{
 */

/** @brief Write a single precision floating point image to a file. */
Error* write_image_f(const Image_f* img, const char* fname);

/*@}*/




/** 
 * @name write_image_with_magick
 *
 * Writes an image to a file using ImageMagick.
 * @{
 */

/** 
 * @brief Write a single precision floating point image to a file using 
 * ImageMagick.
 */
#ifdef JWSC_HAVE_MAGICK
Error* write_image_with_magick_f(const Image_f* img, const char* fname);
#endif

/*@}*/


#ifdef __cplusplus
}
}
#endif


#endif
