/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */

/**
 * @file 
 *
 * @author Kyle Simek
 *
 * @cclicense
 *
 * @brief Declarations for miscellaneous special functions.
 */

#ifndef JWSC_FUNCTION_H
#define JWSC_FUNCTION_H


#include <inttypes.h>


#ifdef __cplusplus
namespace jwsc {
extern "C" {
#endif


/** @brief Computes the log of the gamma function, using the Lanczos approximation. **/
double log_gamma_d(double in);

/** @brief Computes the beta function */
double beta_d(double x, double y);

/** @brief Computes the multinomial beta function */
double mult_beta_d(const double* const x, uint32_t num_elts);

/** @brief Computes the log of the beta function */
double log_beta_d(double x, double y);

/** @brief Computes the log of the multinomial beta function */
double log_mult_beta_d(const double* const x, uint32_t num_elts);


#ifdef __cplusplus
}
}
#endif


#endif
