/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Declarations for matrix convolution operations.
 */


#ifndef MATRIX_CONV_H 
#define MATRIX_CONV_H 


#include <jwsc/config.h>

#include <stdlib.h>

#include "jwsc/base/error.h"
#include "jwsc/matrix/matrix.h"


#ifdef __cplusplus
namespace jwsc {
extern "C" {
#endif


/**
 * @name convolve_matrix
 *
 * Convolves a matrix with a filter.
 * @{
 */

/** @brief Convolves a single precision matrix with a filter. */
Error* convolve_matrix_f
(
    Matrix_f**      m_out, 
    const Matrix_f* m_in, 
    const Matrix_f* h
);

/** @brief Convolves a double precision matrix with a filter. */
Error* convolve_matrix_d
(
    Matrix_d**      m_out, 
    const Matrix_d* m_in, 
    const Matrix_d* h
);

/*@}*/




/**
 * @name convolve_matrix_fft
 *
 * Convolves a matrix with a filter using fft.
 * @{
 */

/** @brief Convolves a single precision matrix with a filter using fft. */
#ifdef JWSC_HAVE_FFTW3F
Error* convolve_matrix_fft_f
(
    Matrix_f**      m_out, 
    const Matrix_f* m_in, 
    const Matrix_f* h
);
#endif

/** @brief Convolves a double precision matrix with a filter using fft. */
#ifdef JWSC_HAVE_FFTW3F
Error* convolve_matrix_fft_d
(
    Matrix_d**      m_out, 
    const Matrix_d* m_in, 
    const Matrix_d* h
);
#endif

/*@}*/


#ifdef __cplusplus
}
}
#endif


#endif
