/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "jwsc/config.h"
#include "jwsc/base/fname.h"

#ifdef JWSC_HAVE_DMALLOC
#include <dmalloc.h>
#endif


int main(int argc, char** argv)
{
    char* s;

    /* get_fname_suffix */
    s = get_fname_suffix("filename.txt");
    assert(strcmp(s,"txt") == 0);
    free(s);

    s = get_fname_suffix("filename.txt.");
    assert(strcmp(s,"") == 0);
    free(s);

    s = get_fname_suffix("filename");
    assert(s == 0);

    /* get_fname_dir */
    s = get_fname_dir("/");
    assert(strcmp(s, "/") == 0);
    free(s);

    s = get_fname_dir("/usr");
    assert(strcmp(s, "/") == 0);
    free(s);

    s = get_fname_dir("/usr/");
    assert(strcmp(s, "/") == 0);
    free(s);

    s = get_fname_dir("/usr/bin/ls");
    assert(strcmp(s, "/usr/bin") == 0);
    free(s);

    s = get_fname_dir("filename.txt");
    assert(strcmp(s, ".") == 0);
    free(s);

    /* get_fname_base */
    s = get_fname_base("/");
    assert(strcmp(s, "/") == 0);
    free(s);

    s = get_fname_base("/usr");
    assert(strcmp(s, "usr") == 0);
    free(s);

    s = get_fname_base("/usr/");
    assert(strcmp(s, "usr") == 0);
    free(s);

    s = get_fname_base("/usr/bin/ls");
    assert(strcmp(s, "ls") == 0);
    free(s);

    s = get_fname_base("filename.txt");
    assert(strcmp(s, "filename.txt") == 0);
    free(s);

    return EXIT_SUCCESS;
}
