/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/matrix/matrix.h"
#include "jwsc/image/image.h"
#include "jwsc/image/image_io.h"
#include "jwsc/image/edge.h"


int main(int argc, char** argv)
{
    float    sigma;
    float    begin_mag_thresh;
    float    end_mag_thresh;
    float    dp_thresh;

    const char* fname_in;
    const char* fname_out;

    Image_f*      img = NULL;
    Matrix_f*     m   = NULL;
    Edge_set_f*   edges = NULL;
    Pixel_f       pxl = {0};
    Error*        e;


    fname_in  = argv[ 1 ];
    fname_out = argv[ 2 ];


    sigma = 1.5;
    begin_mag_thresh = 0.10;
    end_mag_thresh   = 0.05;
    dp_thresh        = 0.95;


    if ((e = read_image_f(&img, fname_in)) != NULL)
    {
        print_error("edge-test", e);
        return EXIT_FAILURE;
    }

    create_matrix_from_image_f(&m, img, 1.0, 1.0, 1.0);

    if ((e = detect_matrix_edge_set_f(&edges, m, sigma, begin_mag_thresh,
                    end_mag_thresh, 0)) != NULL)
    {
        print_error("edge-test", e);
        return EXIT_FAILURE;
    }

    sample_edge_set_f(&edges, edges, 0.9);

    pxl.r = 1.0;
    color_edge_set_f(&img, img, edges, &pxl);

    if ((e = write_image_f(img, fname_out)) != NULL)
    {
        print_error("edge-test", e);
        return EXIT_FAILURE;
    }


    free_matrix_f(m);
    free_image_f(img);
    free_edge_set_f(edges);

    return EXIT_SUCCESS;
}
