/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/image/image.h"


int main(int argc, char** argv)
{
    uint32_t num_rows, num_cols;
    uint32_t row, col;

    Pixel_f   px    = {0.5,0.5,0.5};
    Pixel_f   min   = {0,0,0};
    Pixel_f   max   = {1,1,1};
    Image_f*  img_1 = NULL;
    Image_f*  img_2 = NULL;
    Matrix_f* m_1   = NULL;
    Matrix_f* m_2   = NULL;
    Matrix_f* m_3   = NULL;

    num_rows = 4;
    num_cols = 4;


    create_init_image_f(&img_1, num_rows, num_cols, px);
    fprintf(stdout, "I_1 := (%5f,%5f,%5f)\n", px.r, px.g, px.b);
    for (row = 0; row < img_1->num_rows; row++)
    {
        for (col = 0; col < img_1->num_cols; col++)
        {
            fprintf(stdout, "(%5f,%5f,%5f)  ", img_1->pxls[ row ][ col ].r,
                                               img_1->pxls[ row ][ col ].g,
                                               img_1->pxls[ row ][ col ].b);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    create_random_image_f(&img_1, num_rows, num_cols, min, max);
    fprintf(stdout, "I_1 := rand(%5f,%5f,%5f : %5f,%5f,%5f\n", 
            min.r, min.g, min.b, max.r, max.g, max.b);
    for (row = 0; row < img_1->num_rows; row++)
    {
        for (col = 0; col < img_1->num_cols; col++)
        {
            fprintf(stdout, "(%5f,%5f,%5f)  ", img_1->pxls[ row ][ col ].r,
                                               img_1->pxls[ row ][ col ].g,
                                               img_1->pxls[ row ][ col ].b);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    create_matrices_from_image_f(&m_1, &m_2, &m_3, img_1);
    fprintf(stdout, "(M_1,M_2,M_3) := (I_1.r,I_1.g,I_1.b)\n");
    for (row = 0; row < img_1->num_rows; row++)
    {
        for (col = 0; col < img_1->num_cols; col++)
        {
            fprintf(stdout, "(%5f,%5f,%5f)  ", m_1->elts[ row ][ col ],
                                               m_2->elts[ row ][ col ],
                                               m_3->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");



    create_image_from_matrices_f(&img_2, m_1, m_2, m_3);
    fprintf(stdout, "(I_2.r,I_2.g,I_2.b) := (M_1,M_2,M_3)\n");
    for (row = 0; row < img_1->num_rows; row++)
    {
        for (col = 0; col < img_1->num_cols; col++)
        {
            fprintf(stdout, "(%5f,%5f,%5f)  ", img_2->pxls[ row ][ col ].r,
                                               img_2->pxls[ row ][ col ].g,
                                               img_2->pxls[ row ][ col ].b);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");



    create_matrix_from_image_f(&m_1, img_1, 1, 1, 1);
    fprintf(stdout, "M_1 := avg(I_1.r, I_1.g, I_1.b)\n");
    for (row = 0; row < img_1->num_rows; row++)
    {
        for (col = 0; col < img_1->num_cols; col++)
        {
            fprintf(stdout, "%5f  ", m_1->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");



    create_image_from_matrix_f(&img_2, m_1);
    fprintf(stdout, "(I_2.r,I_2.g,I_2.b) := (M_1,M_1,M_1)\n");
    for (row = 0; row < img_1->num_rows; row++)
    {
        for (col = 0; col < img_1->num_cols; col++)
        {
            fprintf(stdout, "(%5f,%5f,%5f)  ", img_2->pxls[ row ][ col ].r,
                                               img_2->pxls[ row ][ col ].g,
                                               img_2->pxls[ row ][ col ].b);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");



    copy_image_f(&img_2, img_1);
    fprintf(stdout, "I_2 := I_1\n");
    for (row = 0; row < img_1->num_rows; row++)
    {
        for (col = 0; col < img_1->num_cols; col++)
        {
            fprintf(stdout, "(%5f,%5f,%5f)  ", img_2->pxls[ row ][ col ].r,
                                               img_2->pxls[ row ][ col ].g,
                                               img_2->pxls[ row ][ col ].b);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");



    free_matrix_f(m_1);
    free_matrix_f(m_2);
    free_matrix_f(m_3);
    free_image_f(img_1);
    free_image_f(img_2);

    return EXIT_SUCCESS;
}
