/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>
#include <math.h>

#include "jwsc/base/error.h"
#include "jwsc/image/image.h"
#include "jwsc/image/image_io.h"
#include "jwsc/image/jiff.h"


int main(int argc, char** argv)
{
    uint32_t  num_rows, num_cols;
    uint32_t  row, col;
    Pixel_f   min_px = {0, 0, 0};
    Pixel_f   max_px = {1, 1, 1};
    Image_f*  img_1 = NULL;
    Image_f*  img_2 = NULL;
    Error*    e;

    JIFF_attributes attrs;

    num_rows = 50;
    num_cols = 50;

    init_jiff_attributes(&attrs);
    attrs.colorspace = JIFF_RGB;
    attrs.depth      = JIFF_32_BIT;
    attrs.type       = JIFF_FLOAT;

    create_random_image_f(&img_1, num_rows, num_cols, min_px, max_px);

    if ((e = write_image_as_jiff_f(img_1, "img_1.jiff", &attrs)) != NULL)
    {
        print_error_exit("image_io-test", e);
    }

    if ((e = write_image_f(img_1, "img_1.tiff")) != NULL)
    {
        print_error_exit("image_io-test", e);
    }

    if ((e = read_image_f(&img_2, "img_1.jiff")) != NULL)
    {
        print_error_exit("image_io-test", e);
    }

    if ((e = write_image_as_jiff_f(img_2, "img_2.jiff", &attrs)) != NULL)
    {
        print_error_exit("image_io-test", e);
    }

    if ((e = write_image_f(img_2, "img_2.tiff")) != NULL)
    {
        print_error_exit("image_io-test", e);
    }

    assert(img_1->num_rows == img_2->num_rows &&
           img_1->num_cols == img_2->num_cols);

    for (row = 0; row < img_1->num_rows; row++)
    {
        for (col = 0; col < img_1->num_cols; col++)
        {
            assert(fabs(img_1->pxls[row][col].r - 
                        img_2->pxls[row][col].r) < 4.0e-3);
            assert(fabs(img_1->pxls[row][col].g - 
                        img_2->pxls[row][col].g) < 4.0e-3);
            assert(fabs(img_1->pxls[row][col].b - 
                        img_2->pxls[row][col].b) < 4.0e-3);
        }
    }


    free_image_f(img_1);
    free_image_f(img_2);

    return EXIT_SUCCESS;
}
