/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/matrix/matrix.h"
#include "jwsc/image/image.h"
#include "jwsc/image/image_io.h"
#include "jwsc/image/image_util.h"


int main(int argc, char** argv)
{
    float row_factor;
    float col_factor;
    const char* fname_in;
    const char* fname_out;

    Image_f*  img = NULL;
    Matrix_f* mat = NULL;
    Error*   e;


    fname_in  = argv[ 1 ];
    fname_out = argv[ 2 ];

    row_factor = 0.6;
    col_factor = 0.6;


    if ((e = read_image_f(&img, fname_in)) != NULL)
    {
        print_error("threshold-test", e);
        return EXIT_FAILURE;
    }

    create_matrix_from_image_f(&mat, img, 0.3, 0.59, 0.11);
    dynamically_threshold_matrix_f(&mat, mat);
    //threshold_matrix_f(&mat, mat, 0.75);
    create_image_from_matrix_f(&img, mat);

    if ((e = write_image_f(img, fname_out)) != NULL)
    {
        print_error("edge-test", e);
        return EXIT_FAILURE;
    }


    free_matrix_f(mat);
    free_image_f(img);

    if (get_num_unhandled_errors())
    {
        print_error_msg_exit("threshold-test", "Unhandled errors");
    }

    return EXIT_SUCCESS;
}
