/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/imgblock/imgblock.h"


int main(int argc, char** argv)
{
    uint32_t num_imgs, num_rows, num_cols;
    uint32_t img, row, col;

    Pixel_f     px    = {0.5,0.5,0.5};
    Pixel_f     min   = {0,0,0};
    Pixel_f     max   = {1,1,1};
    Imgblock_f* img_1 = NULL;
    Imgblock_f* img_2 = NULL;
    Matblock_f* m_1   = NULL;
    Matblock_f* m_2   = NULL;
    Matblock_f* m_3   = NULL;

    num_imgs = 3;
    num_rows = 4;
    num_cols = 4;


    create_init_imgblock_f(&img_1, num_imgs, num_rows, num_cols, px);
    for (img = 0; img < img_1->num_imgs; img++)
    {
        fprintf(stdout, "I_1[%d] := (%5f,%5f,%5f)\n", img, px.r, px.g, px.b);
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                fprintf(stdout, "(%5f,%5f,%5f)  ", 
                        img_1->pxls[ img ][ row ][ col ].r,
                        img_1->pxls[ img ][ row ][ col ].g,
                        img_1->pxls[ img ][ row ][ col ].b);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");


    create_random_imgblock_f(&img_1, num_imgs, num_rows, num_cols, min, max);
    for (img = 0; img < img_1->num_imgs; img++)
    {
        fprintf(stdout, "I_1[%d] := rand(%5f,%5f,%5f : %5f,%5f,%5f)\n", img,
                min.r, min.g, min.b, max.r, max.g, max.b);
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                fprintf(stdout, "(%5f,%5f,%5f)  ", 
                        img_1->pxls[ img][ row ][ col ].r,
                        img_1->pxls[ img][ row ][ col ].g,
                        img_1->pxls[ img][ row ][ col ].b);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");


    create_matblocks_from_imgblock_f(&m_1, &m_2, &m_3, img_1);
    for (img = 0; img < img_1->num_imgs; img++)
    {
        fprintf(stdout, "(M_1,M_2,M_3)[%d] := (I_1.r,I_1.g,I_1.b)[%d]\n", 
                img, img);
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                fprintf(stdout, "(%5f,%5f,%5f)  ", 
                        m_1->elts[ img ][ row ][ col ],
                        m_2->elts[ img ][ row ][ col ],
                        m_3->elts[ img ][ row ][ col ]);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");



    create_imgblock_from_matblocks_f(&img_2, m_1, m_2, m_3);
    for (img = 0; img < img_1->num_imgs; img++)
    {
        fprintf(stdout, "(I_2.r,I_2.g,I_2.b)[%d] := (M_1,M_2,M_3)[%d]\n", 
                img, img);
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                fprintf(stdout, "(%5f,%5f,%5f)  ", 
                        img_2->pxls[ img ][ row ][ col ].r,
                        img_2->pxls[ img ][ row ][ col ].g,
                        img_2->pxls[ img ][ row ][ col ].b);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");



    create_matblock_from_imgblock_f(&m_1, img_1, 1, 1, 1);
    for (img = 0; img < img_1->num_imgs; img++)
    {
        fprintf(stdout, "M_1[%d] := avg(I_1.r, I_1.g, I_1.b)[%d]\n", img, img);
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                fprintf(stdout, "%5f  ", m_1->elts[ img ][ row ][ col ]);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");



    create_imgblock_from_matblock_f(&img_2, m_1);
    for (img = 0; img < img_1->num_imgs; img++)
    {
        fprintf(stdout, "(I_2.r,I_2.g,I_2.b)[%d] := (M_1,M_1,M_1)[%d]\n", 
                img, img);
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                fprintf(stdout, "(%5f,%5f,%5f)  ", 
                        img_2->pxls[ img ][ row ][ col ].r,
                        img_2->pxls[ img ][ row ][ col ].g,
                        img_2->pxls[ img ][ row ][ col ].b);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");



    copy_imgblock_f(&img_2, img_1);
    for (img = 0; img < img_1->num_imgs; img++)
    {
        fprintf(stdout, "I_2[%d] := I_1[%d]\n", img, img);
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                fprintf(stdout, "(%5f,%5f,%5f)  ", 
                        img_2->pxls[ img ][ row ][ col ].r,
                        img_2->pxls[ img ][ row ][ col ].g,
                        img_2->pxls[ img ][ row ][ col ].b);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");



    free_matblock_f(m_1);
    free_matblock_f(m_2);
    free_matblock_f(m_3);
    free_imgblock_f(img_1);
    free_imgblock_f(img_2);

    return EXIT_SUCCESS;
}
