/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>
#include <math.h>

#include "jwsc/base/error.h"
#include "jwsc/imgblock/imgblock.h"
#include "jwsc/imgblock/imgblock_io.h"


int main(int argc, char** argv)
{
    uint32_t    num_imgs, num_rows, num_cols;
    uint32_t    img, row, col;
    Pixel_f     min_px = {0, 0, 0};
    Pixel_f     max_px = {1, 1, 1};
    Imgblock_f* img_1 = NULL;
    Imgblock_f* img_2 = NULL;
    Error*      e;

    JIFF_attributes attrs;

    const char* fnames[5] = 
        {"img-a.jiff", "img-b.tiff", "img-c.jiff", "img-d.tiff", "img-e.jiff"};

    num_imgs = 5;
    num_rows = 50;
    num_cols = 50;

    init_jiff_attributes(&attrs);
    attrs.colorspace = JIFF_GRAY;
    attrs.depth      = JIFF_32_BIT;
    attrs.type       = JIFF_FLOAT;

    create_random_imgblock_f(&img_1, num_imgs, num_rows, num_cols, min_px, 
            max_px);

    if ((e = write_imgblock_as_numbered_jiff_f(img_1, "img-%04d.jiff", &attrs)) != NULL)
    {
        print_error_exit("imgblock_io-test", e);
    }

    if ((e = write_imgblock_named_f(img_1, fnames)) != NULL)
    {
        print_error_exit("imgblock_io-test", e);
    }

    if ((e = read_imgblock_numbered_f(&img_2, "img-%04d.jiff", num_imgs)) 
            != NULL)
    {
        print_error_exit("imgblock_io-test", e);
    }

    assert(img_1->num_imgs == img_2->num_imgs &&
           img_1->num_rows == img_2->num_rows &&
           img_1->num_cols == img_2->num_cols);

    for (img = 0; img < img_1->num_imgs; img++)
    {
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                assert(fabs(img_1->pxls[img][row][col].r - 
                            img_2->pxls[img][row][col].r) < 4.0e-3);
                assert(fabs(img_1->pxls[img][row][col].g - 
                            img_2->pxls[img][row][col].g) < 4.0e-3);
                assert(fabs(img_1->pxls[img][row][col].b - 
                            img_2->pxls[img][row][col].b) < 4.0e-3);
            }
        }
    }


    if ((e = read_imgblock_named_f(&img_2, fnames, num_imgs)) != NULL)
    {
        print_error_exit("imgblock_io-test", e);
    }

    assert(img_1->num_imgs == img_2->num_imgs &&
           img_1->num_rows == img_2->num_rows &&
           img_1->num_cols == img_2->num_cols);

    for (img = 0; img < img_1->num_imgs; img++)
    {
        for (row = 0; row < img_1->num_rows; row++)
        {
            for (col = 0; col < img_1->num_cols; col++)
            {
                assert(fabs(img_1->pxls[img][row][col].r - 
                            img_2->pxls[img][row][col].r) < 4.0e-3);
                assert(fabs(img_1->pxls[img][row][col].g - 
                            img_2->pxls[img][row][col].g) < 4.0e-3);
                assert(fabs(img_1->pxls[img][row][col].b - 
                            img_2->pxls[img][row][col].b) < 4.0e-3);
            }
        }
    }



    free_imgblock_f(img_1);
    free_imgblock_f(img_2);

    return EXIT_SUCCESS;
}
