/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/imgblock/imgblock.h"
#include "jwsc/imgblock/imgblock_io.h"
#include "jwsc/imgblock/imgblock_util.h"


int main(int argc, char** argv)
{
    uint32_t num_imgs;
    float    img_factor;
    float    row_factor;
    float    col_factor;

    const char* fname_in;
    const char* fname_out;

    Imgblock_f* img = NULL;
    Error*      e;


    fname_in  = argv[ 1 ];
    num_imgs  = strtol(argv[ 2 ], NULL, 10);
    fname_out = argv[ 3 ];

    img_factor = 0.5;
    row_factor = 0.5;
    col_factor = 0.5;


    if ((e = read_imgblock_numbered_f(&img, fname_in, num_imgs)) != NULL)
    {
        print_error("imgblock_util-test", e);
        return EXIT_FAILURE;
    }

    if ((e = down_sample_imgblock_f(&img, img, img_factor, row_factor, 
                    col_factor)) != NULL)
    {
        print_error("imgblock_util-test", e);
        return EXIT_FAILURE;
    }


    if ((e = write_imgblock_numbered_f(img, fname_out)) != NULL)
    {
        print_error("imgblock_util-test", e);
        return EXIT_FAILURE;
    }


    free_imgblock_f(img);

    return EXIT_SUCCESS;
}
