/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/matblock/matblock.h"
#include "jwsc/imgblock/imgblock.h"
#include "jwsc/imgblock/imgblock_io.h"
#include "jwsc/imgblock/surface.h"


int main(int argc, char** argv)
{
    uint32_t num_imgs;
    uint32_t num_pts;
    float    sigma;
    float    begin_thresh;
    float    end_thresh;
    float    grad_img_weight;
    float    grad_row_weight;
    float    grad_col_weight;

    const char* fname_in;
    const char* fname_out;
    const char* pts_fname;

    Imgblock_f*      img = NULL;
    Matblock_f*      m   = NULL;
    Surface_point_f* pts = NULL;
    Pixel_f          pxl = {0};
    Error*           e;


    fname_in  = argv[ 1 ];
    num_imgs  = strtol(argv[ 2 ], NULL, 10);
    fname_out = argv[ 3 ];
    pts_fname = argv[ 4 ];


    sigma = 2.5;
    begin_thresh = 0.06;
    end_thresh   = 0.03;

    grad_img_weight = 3.0;
    grad_row_weight = 1.0;
    grad_col_weight = 1.0;


    if ((e = read_imgblock_numbered_f(&img, fname_in, num_imgs)) != NULL)
    {
        print_error("surface-test", e);
        return EXIT_FAILURE;
    }

    create_matblock_from_imgblock_f(&m, img, 1.0, 1.0, 1.0);

    if ((e = detect_matblock_surface_points_f(&pts, &num_pts, m, sigma, 
                    grad_img_weight, grad_row_weight, grad_col_weight, 
                    begin_thresh, end_thresh)) != NULL)
    {
        print_error("surface-test", e);
        return EXIT_FAILURE;
    }

    sample_surface_points_f(&pts, &num_pts, pts, num_pts, 1.0);

    pxl.r = 1.0;
    color_surface_points_f(&img, img, pts, num_pts, &pxl);

    if ((e = write_imgblock_numbered_f(img, fname_out)) != NULL)
    {
        print_error("surface-test", e);
        return EXIT_FAILURE;
    }

    if ((e = write_surface_points_f(pts, num_pts, pts_fname)) != NULL)
    {
        print_error("surface-test", e);
        return EXIT_FAILURE;
    }


    free_matblock_f(m);
    free_imgblock_f(img);
    free(pts);

    return EXIT_SUCCESS;
}
