/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>

#include "jwsc/matblock/matblock.h"
#include "jwsc/matblock/matblock_math.h"
#include "jwsc/matblock/matblock_fft.h"


int main(int argc, char** argv)
{
    uint32_t num_mats, num_rows, num_cols;
    uint32_t mat, row, col;

    double min, max;

    Matblock_d*  m_1 = NULL;
    Matblock_d*  m_2 = NULL;
    Matblock_cd* m_3 = NULL;

    num_mats = 4;
    num_rows = 4;
    num_cols = 4;
    min      = 0;
    max      = 1;


    srand(4897834);


    create_random_matblock_d(&m_1, num_mats, num_rows, num_cols, min, max);
    for (mat = 0; mat < m_1->num_mats; mat++)
    {
        fprintf(stdout, "M_1[%d] \n", mat);
        for (row = 0; row < m_1->num_rows; row++)
        {
            for (col = 0; col < m_1->num_cols; col++)
            {
                fprintf(stdout, "%e ", m_1->elts[ mat ][ row ][ col ]);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");


    shift_matblock_d(&m_2, m_1);
    fprintf(stdout, "M_2 := shift(M_1) \n");
    for (mat = 0; mat < m_2->num_mats; mat++)
    {
        fprintf(stdout, "M_2[%d] \n", mat);
        for (row = 0; row < m_2->num_rows; row++)
        {
            for (col = 0; col < m_2->num_cols; col++)
            {
                fprintf(stdout, "%e ", m_2->elts[ mat ][ row ][ col ]);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");



#ifdef JWSC_HAVE_FFTW3
    fprintf(stdout, "M_3 := r2c(M_2) \n");
    real_to_complex_matblock_dft_d(&m_3, m_2);
    for (mat = 0; mat < m_3->num_mats; mat++)
    {
        fprintf(stdout, "M_3[%d] \n", mat);
        for (row = 0; row < m_3->num_rows; row++)
        {
            for (col = 0; col < m_3->num_cols; col++)
            {
                fprintf(stdout, "%e i%e ", m_3->elts[ mat ][ row ][ col ].r,
                        m_3->elts[ mat ][ row ][ col ].i);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");


    fprintf(stdout, "M_3 := expanded(M_3) \n");
    expand_symmetric_matblock_dft_d(&m_3, m_3, (m_1->num_cols % 2 == 0));
    for (mat = 0; mat < m_3->num_mats; mat++)
    {
        fprintf(stdout, "M_3[%d] \n", mat);
        for (row = 0; row < m_3->num_rows; row++)
        {
            for (col = 0; col < m_3->num_cols; col++)
            {
                fprintf(stdout, "%e i%e ", m_3->elts[ mat ][ row ][ col ].r,
                        m_3->elts[ mat ][ row ][ col ].i);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");


    fprintf(stdout, "M_3 := shrink(M_3) \n");
    shrink_symmetric_matblock_dft_d(&m_3, m_3);
    for (mat = 0; mat < m_3->num_mats; mat++)
    {
        fprintf(stdout, "M_3[%d] \n", mat);
        for (row = 0; row < m_3->num_rows; row++)
        {
            for (col = 0; col < m_3->num_cols; col++)
            {
                fprintf(stdout, "%e i%e ", m_3->elts[ mat ][ row ][ col ].r,
                        m_3->elts[ mat ][ row ][ col ].i);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");


    fprintf(stdout, "M_2 := c3r(M_3) \n");
    complex_to_real_matblock_dft_d(&m_2, m_3, (m_1->num_cols % 2 == 0));
    scale_matblock_dft_d(&m_2, m_2);
    for (mat = 0; mat < m_2->num_mats; mat++)
    {
        fprintf(stdout, "M_2[%d] \n", mat);
        for (row = 0; row < m_2->num_rows; row++)
        {
            for (col = 0; col < m_2->num_cols; col++)
            {
                fprintf(stdout, "%e ", m_2->elts[ mat ][ row ][ col ]);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");


    unshift_matblock_d(&m_2, m_2);
    fprintf(stdout, "M_2 := unshift(M_2) \n");
    for (mat = 0; mat < m_2->num_mats; mat++)
    {
        fprintf(stdout, "M_2[%d] \n", mat);
        for (row = 0; row < m_2->num_rows; row++)
        {
            for (col = 0; col < m_2->num_cols; col++)
            {
                fprintf(stdout, "%e ", m_2->elts[ mat ][ row ][ col ]);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");


    subtract_matblocks_d(&m_2, m_2, m_1);
    fprintf(stdout, "M_2 := M_2 - M_1 \n");
    for (mat = 0; mat < m_2->num_mats; mat++)
    {
        fprintf(stdout, "M_2[%d] \n", mat);
        for (row = 0; row < m_2->num_rows; row++)
        {
            for (col = 0; col < m_2->num_cols; col++)
            {
                fprintf(stdout, "%e ", m_2->elts[ mat ][ row ][ col ]);
            }
            fprintf(stdout, "\n");
        }
    }
    fprintf(stdout, "\n");
#endif


    free_matblock_d(m_1);
    free_matblock_d(m_2);
    free_matblock_cd(m_3);

    return EXIT_SUCCESS;
}
