/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <jwsc/config.h>

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "jwsc/math/lapack.h"


static const char* usage = "lapack-test <M> <N> <K> [<seed>]";


int main(int argc, const char** argv)
{
    double* A;
    double* work;
    int*    ipiv;
    int     ipiv_len;
    int     M, N;
    int     m, n;
    int     K, k;
    int     info;
    int     seed;

    if (argc < 4)
    {
        fprintf(stderr, "usage: %s\n", usage);
        return EXIT_FAILURE;
    }

    sscanf(argv[1], "%d", &M);
    sscanf(argv[2], "%d", &N);
    sscanf(argv[3], "%d", &K);
    seed = (argc > 4) ? sscanf(argv[4], "%d", &seed) : 0;

    srand(seed);

    assert(A = malloc(M*N*sizeof(double)));
    assert(work = malloc(N*sizeof(double)));

    ipiv_len = (M < N) ? M : N;
    assert(ipiv = malloc(ipiv_len*sizeof(int)));

#if defined JWSC_HAVE_LAPACK
    for (k = 0; k < K; k++)
    {
        fprintf(stdout, "\n");

        fprintf(stdout, "A = \n");
        for (m = 0; m < M; m++)
        {
            for (n = 0; n < N; n++)
            {
                A[ m*N + n ] = (double)rand() / (double)RAND_MAX;
                fprintf(stdout, "%12.9f ", A[ m*N + n ]);
            }
            fprintf(stdout, "\n");
        }

        fprintf(stdout, "\n");

        lapack_dgetrf(M, N, A, M, ipiv, &info);
        assert(info >= 0);
        if (info > 0)
        {
            fprintf(stdout, "Singular\n");
            continue;
        }

        fprintf(stdout, "ipiv = \n");
        for (n = 0; n < ipiv_len; n++)
        {
            fprintf(stdout, " %d", ipiv[ n ]);
        }

        fprintf(stdout, "\n\n");

        fprintf(stdout, "L+U = \n");
        for (m = 0; m < M; m++)
        {
            for (n = 0; n < N; n++)
            {
                fprintf(stdout, "%7.4f ", A[ m*N + n ]);
            }
            fprintf(stdout, "\n");
        }

        fprintf(stdout, "\n");

        if (M == N)
        {
            lapack_dgetri(N, A, N, ipiv, work, N, &info);
            assert(info >= 0);
            if (info > 0)
            {
                fprintf(stdout, "Singular\n");
                continue;
            }

            fprintf(stdout, "A^-1 = \n");
            for (m = 0; m < M; m++)
            {
                for (n = 0; n < N; n++)
                {
                    fprintf(stdout, "%7.4f ", A[ m*N + n ]);
                }
                fprintf(stdout, "\n");
            }

            fprintf(stdout, "\n");
        }
    }
#else
    fprintf(stderr, "No lapack to test\n");
#endif

    free(A);
    free(ipiv);

    return EXIT_SUCCESS;
}
