/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <jwsc/config.h>

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>


double slow_sdot(int n, double* v1, double* v2)
{
    double dot;
    int i;

    dot = 0;
    
    for (i = 0; i < n; i++)
    {
        dot += v1[ i ] * v2[ i ];
    }

    return dot;
}

void slow_saxpy(int n, double a, double* v1, double* v2)
{
    int i;

    for (i = 0; i < n; i++)
    {
        v2[ i ] += a*v1[ i ] + v2[ i ];
    }
}


void slow_dgemv(int n, double* m1, double* v1, double* v2)
{
    int num_rows, num_cols;
    int row, col;
    double sum;

    num_rows = n;
    num_cols = n;

    for (row = 0; row < num_rows; row++)
    {
        sum = 0;
        for (col = 0; col < num_cols; col++)
        {
            sum += m1[ row*num_cols + col ] * v1[ col ];
        }
        v2[ row ] = sum;
    }
}



int main(int argc, char** argv)
{
    double *v1, *v2;
    double *m1;
    int k,n;
    int i,e;

    k = 10000;
    n = 1000;
    srand((unsigned int)384759283);

    assert((v1 = malloc(n*sizeof(double))));
    assert((v2 = malloc(n*sizeof(double))));
    assert((m1 = malloc(n*n*sizeof(double))));

        for (e = 0; e < n; e++)
        {
            v1[ e ] = (double)rand() / (double)RAND_MAX;
            v2[ e ] = (double)rand() / (double)RAND_MAX;
        }

        for (e = 0; e < n*n; e++)
        {
            m1[ e ] = (double)rand() / (double)RAND_MAX;
        }

    for (i = 0; i < k; i++)
    {
        /*slow_sdot(n, v1, v2);*/
        /*slow_saxpy(n, 1.0f, v1, v2);*/
        slow_dgemv(n, m1, v1, v2);
    }

    return EXIT_SUCCESS;
}
