/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/matrix/matrix.h"


int main(int argc, char** argv)
{
    uint32_t num_rows, num_cols;
    uint32_t row, col;

    int64_t min, max;
    int64_t val;
    Matrix_i64* m_1 = NULL;
    Matrix_i64* m_2 = NULL;
    Matrix_i64* m_3 = NULL;

    Error* e;

    num_rows = 5;
    num_cols = 4;

    create_zero_matrix_i64(&m_1, num_rows, num_cols);
    for (row = 0; row < num_rows; row++)
    {
        for (col = 0; col < num_cols; col++)
        {
#ifdef JWSC_HAVE_64_BIT_LONG_INT
            fprintf(stdout, "%lu ", m_1->elts[ row ][ col ]);
#else
            fprintf(stdout, "%llu ", m_1->elts[ row ][ col ]);
#endif
        }
        fprintf(stdout, "\n");
    }

    min = 0;
    max = 255;
    create_random_matrix_i64(&m_2, num_rows, num_cols, min, max);
    for (row = 0; row < num_rows; row++)
    {
        for (col = 0; col < num_cols; col++)
        {
#ifdef JWSC_HAVE_64_BIT_LONG_INT
            fprintf(stdout, "%lu ", m_2->elts[ row ][ col ]);
#else
            fprintf(stdout, "%llu ", m_2->elts[ row ][ col ]);
#endif
        }
        fprintf(stdout, "\n");
    }

    if ((e = copy_matrix_block_i64(&m_3, m_2, 2, 1, 3, 2)) != NULL)
    {
        print_error_exit("matrix-test", e);
    }
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
#ifdef JWSC_HAVE_64_BIT_LONG_INT
            fprintf(stdout, "%lu ", m_3->elts[ row ][ col ]);
#else
            fprintf(stdout, "%llu ", m_3->elts[ row ][ col ]);
#endif
        }
        fprintf(stdout, "\n");
    }

    val = m_3->elts[ 1 ][ 1 ];
#ifdef JWSC_HAVE_64_BIT_LONG_INT
    fprintf(stdout, "%lu\n", val);
#else
    fprintf(stdout, "%llu\n", val);
#endif

    val = 666;
    m_3->elts[ 1 ][ 1 ] = val;
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
#ifdef JWSC_HAVE_64_BIT_LONG_INT
            fprintf(stdout, "%lu ", m_3->elts[ row ][ col ]);
#else
            fprintf(stdout, "%llu ", m_3->elts[ row ][ col ]);
#endif
        }
        fprintf(stdout, "\n");
    }

    free_matrix_i64(m_1);
    free_matrix_i64(m_2);
    free_matrix_i64(m_3);

    return EXIT_SUCCESS;
}
