/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/matrix/matrix.h"
#include "jwsc/matrix/matrix_math.h"
#include "jwsc/matrix/matrix_fft.h"
#include "jwsc/matrix/matrix_conv.h"
#include "jwsc/filter/2d.h"


int main(int argc, char** argv)
{
    uint32_t num_rows, num_cols;
    uint32_t row, col;

    double min, max;

    Matrix_d* m_1 = NULL;
    Matrix_d* m_2 = NULL;
    Matrix_d* m_3 = NULL;
    Matrix_d* m_4 = NULL;

    Error* e;


    num_rows = 8;
    num_cols = 8;
    min      = 0;
    max      = 1;

    create_random_matrix_d(&m_1, num_rows, num_cols, min, max);
    fprintf(stdout, "M_1 \n");
    for (row = 0; row < m_1->num_rows; row++)
    {
        for (col = 0; col < m_1->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_1->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");




    create_auto_2d_gaussian_filter_d(&m_2, 0.35, 0.35);
    normalize_matrix_sum_d(&m_2, m_2);
    fprintf(stdout, "M_2 \n");
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    if ((e = convolve_matrix_d(&m_3, m_1, m_2)) != NULL)
    {
        print_error_exit("matrix_conv-test", e);
    }
    fprintf(stdout, "M_3 := M_1 conv M_2 \n");
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_3->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");




    if ((e = create_2d_gaussian_filter_d(&m_2, 0.5f, 0.5f, num_rows, num_cols))
            != NULL)
    {
        print_error_exit("matrix_conv-test", e);
    }
    normalize_matrix_sum_d(&m_2, m_2);
    fprintf(stdout, "M_2 \n");
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    if ((e = convolve_matrix_d(&m_3, m_1, m_2)) != NULL)
    {
        print_error_exit("matrix_conv-test", e);
    }
    fprintf(stdout, "M_3 := M_1 conv M_2 \n");
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_3->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    shift_matrix_d(&m_2, m_2);
    fprintf(stdout, "M_2 := shift(M_2)\n");
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

#ifdef JWSC_HAVE_FFTW3
    if ((e = convolve_matrix_fft_d(&m_4, m_1, m_2)) != NULL)
    {
        print_error_exit("matrix_conv-test", e);
    }
    fprintf(stdout, "M_4 := M_1 fftconv M_2 \n");
    for (row = 0; row < m_4->num_rows; row++)
    {
        for (col = 0; col < m_4->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_4->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    subtract_matrices_d(&m_4, m_3, m_4);
    fprintf(stdout, "M_4 := M_3 - M_4 \n");
    for (row = 0; row < m_4->num_rows; row++)
    {
        for (col = 0; col < m_4->num_cols; col++)
        {
            fprintf(stdout, "%10.3e ", m_4->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");
#endif


    create_auto_2d_gaussian_dy_filter_d(&m_2, 1.0, 1.0);
    fprintf(stdout, "M_2 \n");
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    create_2d_gaussian_dx_filter_d(&m_2, 1.0, 1.0, 6, 6);
    fprintf(stdout, "M_2 \n");
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    free_matrix_d(m_1);
    free_matrix_d(m_2);
    free_matrix_d(m_3);
    free_matrix_d(m_4);

    return EXIT_SUCCESS;
}
