/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>

#include "jwsc/matrix/matrix.h"
#include "jwsc/matrix/matrix_math.h"
#include "jwsc/matrix/matrix_fft.h"


int main(int argc, char** argv)
{
    uint32_t num_rows, num_cols;
    uint32_t row, col;

    float min, max;

    Matrix_f*  m_1 = NULL;
    Matrix_f*  m_2 = NULL;
    Matrix_f*  m_3 = NULL;
    Matrix_cf* m_4 = NULL;

    num_rows = 4;
    num_cols = 4;
    min      = 0;
    max      = 1;


    create_random_matrix_f(&m_1, num_rows, num_cols, min, max);
    fprintf(stdout, "M_1 \n");
    for (row = 0; row < m_1->num_rows; row++)
    {
        for (col = 0; col < m_1->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_1->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    shift_matrix_f(&m_2, m_1);
    fprintf(stdout, "M_2 := shift(M_1) \n");
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    unshift_matrix_f(&m_3, m_2);
    fprintf(stdout, "M_3 := unshift(M_2) \n");
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_3->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    subtract_matrices_f(&m_3, m_3, m_1);
    fprintf(stdout, "M_3 := M_3 - M_1 \n");
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_3->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


#ifdef JWSC_HAVE_FFTW3
    fprintf(stdout, "M_4 := r2c(M_1) \n");
    real_to_complex_matrix_dft_f(&m_4, m_1);
    for (row = 0; row < m_4->num_rows; row++)
    {
        for (col = 0; col < m_4->num_cols; col++)
        {
            fprintf(stdout, "%e i%e ", m_4->elts[ row ][ col ].r,
                                       m_4->elts[ row ][ col ].i);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    fprintf(stdout, "M_4 := expanded(M_4) \n");
    expand_symmetric_matrix_dft_f(&m_4, m_4, (m_1->num_cols % 2 == 0));
    for (row = 0; row < m_4->num_rows; row++)
    {
        for (col = 0; col < m_4->num_cols; col++)
        {
            fprintf(stdout, "%e i%e ", m_4->elts[ row ][ col ].r,
                                       m_4->elts[ row ][ col ].i);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    fprintf(stdout, "M_4 := shrink(M_4) \n");
    shrink_symmetric_matrix_dft_f(&m_4, m_4);
    for (row = 0; row < m_4->num_rows; row++)
    {
        for (col = 0; col < m_4->num_cols; col++)
        {
            fprintf(stdout, "%e i%e ", m_4->elts[ row ][ col ].r,
                                       m_4->elts[ row ][ col ].i);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    fprintf(stdout, "M_3 := c2r(M_4) \n");
    complex_to_real_matrix_dft_f(&m_3, m_4, (m_1->num_cols % 2 == 0));
    scale_matrix_dft_f(&m_3, m_3);
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_3->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");


    subtract_matrices_f(&m_3, m_3, m_1);
    fprintf(stdout, "M_3 := M_3 - M_1 \n");
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_3->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");
#endif


    free_matrix_f(m_1);
    free_matrix_f(m_2);
    free_matrix_f(m_3);
    free_matrix_cf(m_4);

    return EXIT_SUCCESS;
}
