/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/base/file_io.h"
#include "jwsc/matrix/matrix.h"
#include "jwsc/matrix/matrix_io.h"


int main(int argc, char** argv)
{
    uint32_t     num_rows, num_cols;
    uint32_t     row, col;
    int64_t      min_i64, max_i64;
    double       min_d, max_d;
    Matrix_i64*  m_1 = NULL;
    Matrix_d*    m_2 = NULL;
    Error*       e;

    num_rows = 5;
    num_cols = 4;

    min_i64 = -100;
    max_i64 =  100;
    create_random_matrix_i64(&m_1, num_rows, num_cols, min_i64, max_i64);
    for (row = 0; row < m_1->num_rows; row++)
    {
        for (col = 0; col < m_1->num_cols; col++)
        {
#ifdef JWSC_HAVE_64_BIT_LONG_INT
            fprintf(stdout, "%ld ", m_1->elts[ row ][ col ]);
#else
            fprintf(stdout, "%lld ", m_1->elts[ row ][ col ]);
#endif
        }
        fprintf(stdout, "\n");
    }
    if ((e = write_matrix_with_header_i64(m_1, "m_1")) != NULL)
    {
        print_error_exit("matrix_io-test", e);
    }
    if ((e = read_matrix_with_header_i64(&m_1, "m_1")) != NULL)
    {
        print_error_exit("matrix_io-test", e);
    }
    for (row = 0; row < m_1->num_rows; row++)
    {
        for (col = 0; col < m_1->num_cols; col++)
        {
#ifdef JWSC_HAVE_64_BIT_LONG_INT
            fprintf(stdout, "%ld ", m_1->elts[ row ][ col ]);
#else
            fprintf(stdout, "%lld ", m_1->elts[ row ][ col ]);
#endif
        }
        fprintf(stdout, "\n");
    }

    min_d = -1;
    max_d =  1;
    create_random_matrix_d(&m_2, num_rows, num_cols, min_d, max_d);
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%f ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    if ((e = write_matrix_with_header_d(m_2, "m_2")) != NULL)
    {
        print_error_exit("matrix_io-test", e);
    }
    if ((e = read_matrix_with_header_d(&m_2, "m_2")) != NULL)
    {
        print_error_exit("matrix_io-test", e);
    }
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%f ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }

    free_matrix_i64(m_1);
    free_matrix_d(m_2);

    return EXIT_SUCCESS;
}
