/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/vector/vector.h"
#include "jwsc/matrix/matrix.h"
#include "jwsc/matrix/matrix_math.h"


int main(int argc, char** argv)
{
    uint32_t num_rows, num_cols;
    uint32_t row, col;

    float min, max;
    float v_min, v_max;
    float val;

    Matrix_f* m_1 = NULL;
    Matrix_f* m_2 = NULL;
    Matrix_f* m_3 = NULL;
    Vector_f* v_1 = NULL;

    Error* e;

    num_rows = 5;
    num_cols = 4;
    min      = 0;
    max      = 1;

    create_random_matrix_f(&m_1, num_rows, num_cols, min, max);
    fprintf(stdout, "M_1 \n");
    for (row = 0; row < m_1->num_rows; row++)
    {
        for (col = 0; col < m_1->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_1->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    val = 0.5;
    add_scalar_to_matrix_f(&m_1, m_1, val);
    fprintf(stdout, "M_1 := %e .+ M_1 \n", val);
    for (row = 0; row < m_1->num_rows; row++)
    {
        for (col = 0; col < m_1->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_1->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    val = -1.0;
    multiply_matrix_by_scalar_f(&m_1, m_1, val);
    fprintf(stdout, "M_1 := %e .* M_1 \n", val);
    for (row = 0; row < m_1->num_rows; row++)
    {
        for (col = 0; col < m_1->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_1->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    transpose_matrix_f(&m_1, m_1);
    fprintf(stdout, "M_1 := M_1' \n");
    for (row = 0; row < m_1->num_rows; row++)
    {
        for (col = 0; col < m_1->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_1->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    create_random_matrix_f(&m_2, num_rows, 3, min, max);
    fprintf(stdout, "M_2 \n");
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    if ((e = multiply_matrices_f(&m_2, m_1, m_2)) != NULL)
    {
        print_error_exit("matrix_util-test", e);
    }
    fprintf(stdout, "M_2 := M_1 * M_2 \n");
    for (row = 0; row < m_2->num_rows; row++)
    {
        for (col = 0; col < m_2->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");

    create_random_matrix_f(&m_3, num_rows, num_cols, min, max);
    fprintf(stdout, "M_3 \n");
    for (row = 0; row < m_3->num_rows; row++)
    {
        for (col = 0; col < m_3->num_cols; col++)
        {
            fprintf(stdout, "%e ", m_3->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }
    fprintf(stdout, "\n");
    v_min = 0;
    v_max = 1;
    create_random_vector_f(&v_1, num_cols, v_min, v_max);
    fprintf(stdout, "V_1 \n");
    for (col = 0; col < v_1->num_elts; col++)
    {
        fprintf(stdout, "%e ", v_1->elts[ col ]);
    }
    fprintf(stdout, "\n\n");
    if ((e = multiply_matrix_and_vector_f(&v_1, m_3, v_1)) != NULL)
    {
        print_error_exit("matrix_util-test", e);
    }
    fprintf(stdout, "M_3 * V_1' \n");
    for (col = 0; col < v_1->num_elts; col++)
    {
        fprintf(stdout, "%e ", v_1->elts[ col ]);
    }
    fprintf(stdout, "\n\n");

    free_matrix_f(m_1);
    free_matrix_f(m_2);
    free_matrix_f(m_3);
    free_vector_f(v_1);

    return EXIT_SUCCESS;
}
