/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/matrix/matrix.h"
#include "jwsc/matrix/matrix_math.h"


static const char* usage = "pseudoinvert-test <M> <N> [<seed>]";


int main(int argc, const char** argv)
{
    Matrix_d* M1 = NULL;
    Matrix_d* M2 = NULL;
    Matrix_d* I  = NULL;
    Error*    err;

    uint32_t num_rows, num_cols;
    uint32_t row, col;
    uint32_t seed;

    float r;

#if defined JWSC_HAVE_LAPACK
    if (argc < 3)
    {
        fprintf(stderr, "usage: %s\n", usage);
        return EXIT_FAILURE;
    }

    sscanf(argv[1], "%u", &num_rows);
    sscanf(argv[2], "%u", &num_cols);
    if (argc > 3)
    {
        sscanf(argv[3], "%u", &seed);
    }
    else
    {
        seed = 0;
    }

    srand(seed);

    create_random_matrix_d(&M1, num_rows, num_cols, 0, 1);

    fprintf(stdout, "\n");

    fprintf(stdout, "M1 = \n");
    for (row = 0; row < num_rows; row++)
    {
        for (col = 0; col < num_cols; col++)
        {
            r = (double)rand() / RAND_MAX;
            M1->elts[ row ][ col ] = r;
            fprintf(stdout, "%9.6f ", r);
        }
        fprintf(stdout, "\n");
    }

    fprintf(stdout, "\n");

    if ((err = pseudoinvert_matrix_d(&M2, M1)) != NULL)
    {
        print_error_msg_exit(NULL, err->msg);
    }

    fprintf(stdout, "\n");

    fprintf(stdout, "M2 = \n");
    for (row = 0; row < M2->num_rows; row++)
    {
        for (col = 0; col < M2->num_cols; col++)
        {
            fprintf(stdout, "%9.6f ", M2->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }

    fprintf(stdout, "\n");

    assert(multiply_matrices_d(&I, M1, M2) == NULL);

    fprintf(stdout, "M1 * M2 = \n");
    for (row = 0; row < I->num_rows; row++)
    {
        for (col = 0; col < I->num_cols; col++)
        {
            fprintf(stdout, "%9.6f ", I->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }

    fprintf(stdout, "\n");

    free_matrix_d(M1);
    free_matrix_d(M2);
    free_matrix_d(I);
#else
    fprintf(stderr, "No lapack\n");
#endif

    return EXIT_SUCCESS;
}
