/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>

#include "jwsc/base/limits.h"
#include "jwsc/vector/vector.h"
#include "jwsc/prob/pdf.h"


static const char* usage = "gamma-test <N> [<seed>]";


int main(int argc, const char** argv)
{
    double alpha;
    double beta;
    Vector_d* x  = NULL;
    Vector_d* p  = NULL;
    Vector_d* lp = NULL;

    uint32_t n;
    uint32_t N;
    uint32_t seed;

    if (argc < 2)
    {
        fprintf(stderr, "usage: %s\n", usage);
        return EXIT_FAILURE;
    }

    sscanf(argv[1], "%u", &N);
    if (argc > 2)
    {
        sscanf(argv[2], "%u", &seed);
    }
    else
    {
        seed = 0;
    }

    srand(seed);

    alpha = 6;
    beta = 2;

    fprintf(stdout, "\n");

    fprintf(stdout, "alpha = %6.3f\n", alpha);
    fprintf(stdout, "beta = %6.3f\n", beta);

    create_vector_d(&x, N);
    create_vector_d(&p, N);
    create_vector_d(&lp, N);
    for (n = 0; n < N; n++)
    {
        x->elts[ n ] = sample_gamma_pdf_d(alpha, beta, 1.0e-16, 
                6.0*sqrt(alpha)/beta);
        p->elts[ n ] = gamma_pdf_d(alpha, beta, x->elts[ n ]);
        lp->elts[ n ] = log_gamma_pdf_d(alpha, beta, x->elts[ n ]);
    }

    fprintf(stdout, "\n");

    for (n = 0; n < N; n++)
    {
        fprintf(stdout, "%6.3f  %10e  %6.3f\n", x->elts[ n ], p->elts[ n ], 
                lp->elts[ n ]);
    }

    free_vector_d(lp);
    free_vector_d(p);
    free_vector_d(x);

    return EXIT_SUCCESS;
}
