/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "jwsc/matrix/matrix.h"
#include "jwsc/matrix/matrix_io.h"
#include "jwsc/vector/vector.h"
#include "jwsc/stat/kmeans.h"


int main(int argc, char** argv)
{
    Matrix_d*   data_1 = NULL;
    Matrix_d*   data_2 = NULL;
    Matrix_d*   data_3 = NULL;
    Matrix_d*   data   = NULL;
    Matrix_d*   means  = NULL;
    Vector_i32* members = NULL;

    uint32_t row, col;
    
    create_random_matrix_d(&data_1, 10, 3, 0, 1);
    create_random_matrix_d(&data_2, 10, 3, 4, 5);
    create_random_matrix_d(&data_3, 10, 3, 8, 9);
    create_matrix_d(&data, 30, 3);
    copy_matrix_block_into_matrix_d(data, 0, 0, data_1, 0, 0, 10, 3);
    copy_matrix_block_into_matrix_d(data, 10, 0, data_2, 0, 0, 10, 3);
    copy_matrix_block_into_matrix_d(data, 20, 0, data_3, 0, 0, 10, 3);
    free_matrix_d(data_1);
    free_matrix_d(data_2);
    free_matrix_d(data_3);

    srand(29304687);

    init_kmeans_randomly_d(&means, data, 3);

    write_matrix_fp_d(data, stdout);
    fprintf(stdout, "\n");
    write_matrix_fp_d(means, stdout);

    kmeans_d(&means, &members, data, 3, 1);

    fprintf(stdout, "\n");
    write_matrix_fp_d(means, stdout);

    fprintf(stdout, "\n");
    for (row = 0; row < data->num_rows; row++)
    {
        fprintf(stdout, "%d  ", members->elts[ row ]);
        for (col = 0; col < data->num_cols; col++)
        {
            fprintf(stdout, "%lf ", data->elts[ row ][ col ]);
        }
        fprintf(stdout, "\n");
    }

    free_matrix_d(data);
    free_matrix_d(means);
    free_vector_i32(members);

    return EXIT_SUCCESS;
}
