/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>

#include "jwsc/base/error.h"
#include "jwsc/base/file_io.h"
#include "jwsc/math/complex.h"
#include "jwsc/vector/vector.h"
#include "jwsc/vector/vector_io.h"


int main(int argc, char** argv)
{
    uint32_t     num_elts;
    uint32_t     elt;
    int64_t      min_i64, max_i64;
    double       min_d, max_d;
    Complex_d    min_cd, max_cd;
    Vector_i64*  v_1 = NULL;
    Vector_d*    v_2 = NULL;
    Vector_cd*   v_3 = NULL;
    Error*       e;

    num_elts = 5;

    min_i64 = -100;
    max_i64 =  100;
    create_random_vector_i64(&v_1, num_elts, min_i64, max_i64);
    for (elt = 0; elt < v_1->num_elts; elt++)
    {
#ifdef JWSC_HAVE_64_BIT_LONG_INT
        fprintf(stdout, "%ld ", v_1->elts[ elt ]);
#else
        fprintf(stdout, "%lld ", v_1->elts[ elt ]);
#endif
    }
    fprintf(stdout, "\n");
    if ((e = write_vector_with_header_i64(v_1, "v_1")) != NULL)
    {
        print_error_exit("vector_io-test", e);
    }
    if ((e = read_vector_with_header_i64(&v_1, "v_1")) != NULL)
    {
        print_error_exit("vector_io-test", e);
    }
    for (elt = 0; elt < v_1->num_elts; elt++)
    {
#ifdef JWSC_HAVE_64_BIT_LONG_INT
        fprintf(stdout, "%ld ", v_1->elts[ elt ]);
#else
        fprintf(stdout, "%lld ", v_1->elts[ elt ]);
#endif
    }
    fprintf(stdout, "\n");

    min_d = -1;
    max_d =  1;
    create_random_vector_d(&v_2, num_elts, min_d, max_d);
    for (elt = 0; elt < v_2->num_elts; elt++)
    {
        fprintf(stdout, "%f ", v_2->elts[ elt ]);
    }
    fprintf(stdout, "\n");
    if ((e = write_vector_with_header_d(v_2, "v_2")) != NULL)
    {
        print_error_exit("vector_io-test", e);
    }
    if ((e = read_vector_with_header_d(&v_2, "v_2")) != NULL)
    {
        print_error_exit("vector_io-test", e);
    }
    for (elt = 0; elt < v_2->num_elts; elt++)
    {
        fprintf(stdout, "%f ", v_2->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    num_elts = 3;
    min_cd.r = 0;
    min_cd.i = 0;
    max_cd.r = 1;
    max_cd.i = 1;
    create_random_vector_cd(&v_3, num_elts, min_cd, max_cd);
    for (elt = 0; elt < v_3->num_elts; elt++)
    {
        fprintf(stdout, "%lf i%lf ", v_3->elts[ elt ].r, v_3->elts[ elt ].i);
    }
    fprintf(stdout, "\n");
    if ((e = write_vector_with_header_cd(v_3, "v_3")) != NULL)
    {
        print_error_exit("vector_io-test", e);
    }
    if ((e = read_vector_with_header_cd(&v_3, "v_3")) != NULL)
    {
        print_error_exit("vector_io-test", e);
    }
    for (elt = 0; elt < v_3->num_elts; elt++)
    {
        fprintf(stdout, "%lf i%lf ", v_3->elts[ elt ].r, v_3->elts[ elt ].i);
    }
    fprintf(stdout, "\n");

    free_vector_i64(v_1);
    free_vector_d(v_2);
    free_vector_cd(v_3);

    return EXIT_SUCCESS;
}
