/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <inttypes.h>

#include "jwsc/vector/vector.h"
#include "jwsc/vector/vector_math.h"


int main(int argc, char** argv)
{
    uint32_t num_elts;
    uint32_t elt;

    float min, max;
    float val;
    Vector_f* v_1 = NULL;
    Vector_f* v_2 = NULL;
    Vector_f* v_3 = NULL;

    num_elts = 5;
    min      = 0;
    max      = 1;

    create_random_vector_f(&v_1, num_elts, min, max);
    for (elt = 0; elt < v_1->num_elts; elt++)
    {
        fprintf(stdout, "%e ", v_1->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    val = 0.5;
    add_scalar_to_vector_f(&v_2, v_1, val);
    for (elt = 0; elt < v_2->num_elts; elt++)
    {
        fprintf(stdout, "%e ", v_2->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    val = -1.0;
    multiply_vector_by_scalar_f(&v_2, v_2, val);
    for (elt = 0; elt < v_2->num_elts; elt++)
    {
        fprintf(stdout, "%e ", v_2->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    normalize_vector_sum_f(&v_2, v_1);
    for (elt = 0; elt < v_2->num_elts; elt++)
    {
        fprintf(stdout, "%e ", v_2->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    calc_vector_asum_f(&val, v_1);
    fprintf(stdout, "%f\n", val);
    calc_vector_asum_f(&val, v_2);
    fprintf(stdout, "%f\n", val);

    normalize_vector_mag_f(&v_2, v_1);
    for (elt = 0; elt < v_2->num_elts; elt++)
    {
        fprintf(stdout, "%e ", v_2->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    calc_vector_mag_f(&val, v_1);
    fprintf(stdout, "%f\n", val);
    calc_vector_mag_f(&val, v_2);
    fprintf(stdout, "%f\n", val);

    add_vectors_f(&v_3, v_2, v_1);
    for (elt = 0; elt < v_3->num_elts; elt++)
    {
        fprintf(stdout, "%e ", v_3->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    subtract_vectors_f(&v_3, v_3, v_2);
    for (elt = 0; elt < v_3->num_elts; elt++)
    {
        fprintf(stdout, "%e ", v_3->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    calc_vector_dot_prod_f(&val, v_1, v_1);
    fprintf(stdout, "%f\n", val);
    calc_vector_mag_f(&val, v_1);
    fprintf(stdout, "%f\n", val*val);

    create_zero_vector_f(&v_1, 3);
    create_zero_vector_f(&v_2, 3);

    v_1->elts[ 1 ] = 1;
    v_2->elts[ 0 ] = 1;

    calc_3d_vector_cross_prod_f(&v_3, v_1, v_2);
    for (elt = 0; elt < v_3->num_elts; elt++)
    {
        fprintf(stdout, "%f ", v_3->elts[ elt ]);
    }
    fprintf(stdout, "\n");

    create_init_vector_f(&v_1, 2, 1);
    float mf;
    calc_vector_mag_f(&mf, v_1);
    for (elt = 0; elt < v_1->num_elts; elt++)
    {
        fprintf(stdout, "%f ", v_1->elts[ elt ]);
    }
    fprintf(stdout, "\n");
    fprintf(stdout, "%f\n", mf);

    free_vector_f(v_1);
    free_vector_f(v_2);
    free_vector_f(v_3);

    return EXIT_SUCCESS;
}
